<?php
/*
 * Copyright (C) 2009 MDBitz - Matthew John Denton - mdbitz.com
 *
 * This file is part of YahooFinanceAPI.
 *
 * YahooFinanceAPI is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YahooFinanceAPI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YahooFinanceAPI.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * Options
 *
 * This file contains the class YahooFinance_Options
 *
 * @author Matthew Denton <matt@mdbitz.com>
 * @package com.mdbitz.YahooFinance
 */

/**
 * YahooFinance_Options defines the options requested via the API, as well
 * as handlers to parse resulting data to Quote option.
 *
 * @package com.mdbitz.YahooFinance
 */
class YahooFinance_Options
{

    /**
     * @var array option variables
     */
    public static $_SpecialTags = array(
                              "ask" => "a",
                              "averageDailyVolume" => "a2",
                              "askSize" => "a5",
                              "bid" => "b",
                              "askRealTime" => "b2",
                              "bidRealTime" => "b3",
                              "bookValue" => "b4",
                              "bidSize" => "b6",
                              "changeAndPercentChange" => "c",
                              "change" => "c1",
                              "commision" => "c3",
                              "changeRealTime" => "c6",
                              "afterHoursChangeRealTime" => "c8",
                              "dividendPerShare" => "d",
                              "lastTradeDate" => "d1",
                              "tradeDate" => "d2",
                              "earningsPerShare" => "e",
                              "errorIndication" => "e1",
                              "EPSEstimateCurrentYear" => "e7",
                              "EPSEstimateNextYear" => "e8",
                              "EPSEstimateNextQuarter" => "e9",
                              "floatShares" => "f6",
                              "daysLow" => "g",
                              "daysHigh" => "h",
                              "52WeekLow" => "j",
                              "52WeekHigh" => "k",
                              "holdingsGainPercent" => "g1",
                              "annualizedGain" => "g3",
                              "holdingsGain" => "g4",
                              "holdingsGainPercentRealTime" => "g5",
                              "holdingsGainRealTime" => "g6",
                              "moreInfo" => "i",
                              "orderBookRealTime" => "i5",
                              "marketCapitalization" => "j1",
                              "marketCapitalizationRealTime" => "j3",
                              "EBITDA" => "j4",
                              "changeFrom52WeekLow" => "j5",
                              "percentChangeFrom52WeekLow" => "j6",
                              "lastTradeWithTimeRealTime" => "k1",
                              "changePercentRealTime" => "k2",
                              "lastTradeSize" => "k3",
                              "changeFrom52WeekHigh" => "k4",
                              "percentChangeFrom52WeekHigh" => "k5",
                              "lastTradeWithTime" => "l",
                              "lastTrade" => "l1",
                              "highLimit" => "l2",
                              "lowLimit" => "l3",
                              "daysRange" => "m",
                              "daysRangeRealTime" => "m2",
                              "50DayMovingAverage" => "m3",
                              "200DayMovingAverage" => "m4",
                              "changeFrom200DayMovingAverage" => "m5",
                              "percentChangeFrom200DayMovingAverage" => "m6",
                              "changeFrom50DayMovingAverage" => "m7",
                              "percentChangeFrom50DayMovingAverage" => "m8",
                              "name" => "n",
                              "notes" => "n4",
                              "open" => "o",
                              "previousClose" => "p",
                              "pricePaid" => "p1",
                              "changeInPercent" => "c2",
                              "pricePerSales" => "p5",
                              "pricePerBook" => "p6",
                              "exDividendDate" => "q",
                              "priceEarningsRatio" => "r",
                              "dividendPayDate" => "r1",
                              "priceEarningsRatioRealTime" => "r2",
                              "PEGRatio" => "r5",
                              "pricePerEPSEstimateCurrentYear" => "r6",
                              "pricePerEPSEstimateNextYear" => "r7",
                              "symbol" => "s",
                              "sharesOwned" => "s1",
                              "shortRatio" => "s7",
                              "lastTradeTime" => "t1",
                              "tradeLinks" => "t6",
                              "tickerTrend" => "t7",
                              "1YrTargetPrice" => "t8",
                              "volume" => "v",
                              "holdingsValue" => "v1",
                              "holdingsValueRealTime" => "v7",
                              "52WeekRange" => "w",
                              "daysValueChange" => "w1",
                              "daysValueChangeRealTime" => "w4",
                              "stockExchange" => "x",
                              "dividendYeild" => "y"
                              );

    /**
     * @var array Quote Options
     */    
    protected $_options = array();

    /**
     * Constructor initializes options
     *
     * @param array $options quote options
     */
    public function __construct( Array $options = null)
    {   
        if( ! is_null( $options ) ) {
            foreach( $options as $key => $value ) {
                $this->set( $key, $value );
            }
        }
    }

    /**
     * magic method to return non public properties
     *
     * @see     get
     * @param   mixed $property
     * @return  mixed
     */
    public function __get( $property )
    {
        return $this->get( $property );
    }

    /**
     * Return the specified property
     *
     * @param mixed $property     The property to return
     * @return mixed
     */
    public function get( $property )
    {
        if( array_key_exists( $property, YahooFinance_Options::$_SpecialTags ) ) {
            if( array_key_exists( $property, $this->_options ) ){
                return $this->_values[$property];
            } else {
                return null;
            }
        } else {
            throw new YahooFinance_Exception(sprintf('Unknown property %s::%s', get_class($this), $property));
        }
    }

    /**
     * magic method to set non public properties
     *
     * @see    set
     * @param  mixed $property
     * @param  mixed $value
     * @return void
     */
    public function __set( $property, $value )
    {
        $this->set( $property, $value );
    }

    /**
     * sets the specified property
     *
     * @param mixed $property     The property to set
     * @param mixed $value        value of property
     * @return void
     */
    public function set( $property, $value )
    {
        if( array_key_exists( $property, YahooFinance_Options::$_SpecialTags ) ){
            if( is_bool( $value ) ){
                $this->_options[$property] = $value;
            } else {
                throw new YahooFinance_Exception(sprintf("Property needs to be set to a boolean [true,false] %s::%s", get_class($this), $property));
            }
        } else {
            throw new YahooFinance_Exception(sprintf('Unknown property %s::%s', get_class($this), $property));
        }
    }

    /**
     * generated paramater string of user requested options.
     *
     * @return string
     */
    public function toParamString()
    {
        $paramStr = "";
        foreach( $this->_options as $key => $value ){
            if( $value ) {
                $paramStr .= YahooFinance_Options::$_SpecialTags[$key];
            }
        }
        return $paramStr;
    }

    /**
     * converts returned data into quote objects
     *
     * @param string $csvStrInput response data
     * @return array Quote Objects
     */
    public function parseResult( $csvStrInput, $delimiter )
    {
        $results = array();
        $csvStrs = split("[\n|\r]", $csvStrInput);
        foreach( $csvStrs as $csvStr ){
            if( !empty( $csvStr ) ) {
                $csvArray = explode($delimiter, $csvStr );
                $propPos = 0;
                $quote = new YahooFinance_Quote();
                foreach( $this->_options as $key => $value ) {
                    $propVal = $csvArray[$propPos];
                    $propVal = trim( $propVal );
                    if( 0 === strpos($propVal, "\"") ) {
                        $propVal = substr( $propVal, 1, strlen( $propVal ) - 2 );
                    }
					// values different with ; delimiter
					if( $delimiter == ";" ) {
						if( $key == "lastTradeTime" ) {
							$propPos ++;
							$propVal .= " ". $csvArray[$propPos];
						} else if( $key == "lastTradeDate" ) {
							$propPos --;
							$propVal = "";
						}
					}
					
                    $quote->set( $key, $propVal );
                    $propPos ++;
                }
                $results[] = $quote;
            }
        }
        return $results;
    }

}